import { _decorator, Component, director, FogInfo, Touch, Vec3,Node, tween, Tween, Vec2 } from 'cc';
const { ccclass, property } = _decorator;
const tempV3_0 = new Vec3();
const tempV3_1 = new Vec3();
const tempV2_0 = new Vec2();

@ccclass('fogMoveCtrl')
export class fogMoveCtrl extends Component {
    @property speed=10;
    /* change the range of fog */
    @property ({slide:true,range:[0,1000],step:0.1})
    set range(v){
        this._range = v;
        this.initFog();
    }
    get range(){
        return this._range;
    }
    /* change the density of fog */
    @property ({slide:true,range:[0,1],step:0.001})
    set density(v){
        director.getScene().globals.fog.fogDensity = v;
    }
    get density(){
        return director.getScene().globals.fog.fogDensity;
    }

    @property({visible:false})
    _range = 150;

    _fog:FogInfo; 
    _move = false; /* move flag */
    _dir = 1; /* direction */

    onEnable() {
        this.initFog();
        const canvas = director.getScene().getChildByName("Canvas");
        canvas.on(Node.EventType.TOUCH_END,this.moveChar,this);
    }

    /* init fog properties */
    initFog(){
        if(!this._fog){
            this._fog=director.getScene().globals.fog;
        }
        this._fog.fogRange = this._range;
        tempV3_1.set(this.node.worldPosition);
        this._fog.fogStart = tempV3_1.x;
        this._fog.fogEnd = tempV3_1.y;
    }

    onDisable() {
        const canvas = director.getScene().getChildByName("Canvas");
        canvas.off(Node.EventType.TOUCH_END,this.moveChar,this);
        Tween.stopAllByTarget(this.node);
    }

    /* use tween to move char */
    moveChar(event:Touch){
        tempV3_1.x = this.node.worldPosition.x;
        tempV3_1.y = this.node.worldPosition.y;
        event.getUILocation(tempV2_0);
        tempV3_0.x = tempV2_0.x;
        tempV3_0.y = tempV2_0.y;
        const length = Vec3.distance(tempV3_0,tempV3_1);
        if(length>5){
            const direction = (tempV3_1.x-tempV3_0.x)>0?1:-1;
            if(this._dir!=direction){
                this.node.setScale(direction,1,1);
                this._dir=direction;
            }
            this._move = true;
            const time = length/this.speed;
            Tween.stopAllByTarget(this.node);
            tween(this.node).to(time,{worldPosition:tempV3_0}).call(()=>{
                this._move = false;
            }).start();
        }
    }
    /* change the fog start and end once moved */
    update(deltaTime: number) {
        if(this._move){
            tempV3_1.set(this.node.worldPosition);
            this._fog.fogStart = tempV3_1.x;
            this._fog.fogEnd = tempV3_1.y;
        }
    }
}

